#!c:/perl/bin/perl

use strict;
use Config;
use XML::Simple;
use Mysql;
use Net::Syslog;
use Net::SMTP;
use File::Basename;

  sub logEvent {


	# Valid Facilities are:  
	#  kern, user, mail, daemon, auth, syslog, lpr, news, uucp, cron,  
	#  authpriv, ftp, local0, local1, local2, local3, local4, local5, local6 

	# Valid Priorities are:  
	#  emerg, alert, crit, err, warning, notice, info, debug 

	my ($aloe, $management_ip, $event_source, $ticket_status, $event_message, $event_priority) = @_;

  	my $debug	  = $aloe->{settings}->{debug}->{value};
	my $log_message	  =  "-> $management_ip $event_message";

	$event_source = basename($event_source);


	  if (($aloe->{settings}->{aloeSysLogMethod}->{value}) && ($ticket_status ne "Update")) {

		my $log_message =  "-> $management_ip $event_message";

		my $s=new Net::Syslog(Facility=>$aloe->{settings}->{aloeLogFacility}->{value}, Priority=>'debug', SyslogHost=>$aloe->{settings}->{aloeLogHost}->{value}, Name=>$event_source);
		$s->send($log_message,Priority=>$event_priority);

		if ($debug) { print "\n Syslog message has been generated.\n"; }

	  }


	  if (($aloe->{settings}->{aloeDBLogMethod}->{value}) && ($ticket_status ne "Update")) {

		my $syslog_facility	= $aloe->{settings}->{aloeLogFacility}->{value};
		my $aloedbhost		= $aloe->{settings}->{aloeDBHost}->{value};
		my $aloedb		= $aloe->{settings}->{aloeDB}->{value};
		my $aloedbuser		= $aloe->{settings}->{aloeDBUser}->{value};
		my $aloedbpswd		= $aloe->{settings}->{aloeDBPswd}->{value};

		my $log_message =  "$event_source -> $management_ip $event_message";	

	  	my $dbquery = "insert into logs (date, time, priority, facility, host, msg, program) values (now(), now(), '$event_priority', '$syslog_facility', 'localhost', '$log_message', '$event_source')"; 

	  	my $adbh = Mysql->connect(undef, $aloedb, $aloedbuser, $aloedbpswd);
	  	my $adbh = Mysql->connect($aloedbhost, $aloedb, $aloedbuser, $aloedbpswd);

	  	my $asth = $adbh->query($dbquery);

  	  }


  	if (($aloe->{settings}->{aloeSMTPLogMethod}->{value}) && ($ticket_status ne "Update"))  {

	  	my $log_message =  "$event_source -> $management_ip $event_message";

	   	use Net::SMTP;
	    	my $smtp = Net::SMTP->new($aloe->{settings}->{aloeSMTPRelay}->{value});
	    	$smtp->mail($aloe->{settings}->{aloeSMTPFromAddress}->{value});
	    	$smtp->to($aloe->{settings}->{aloeSMTPRecipient}->{value});
	    	$smtp->data();
	    	$smtp->datasend("To: $aloe->{settings}->{aloeSMTPRecipient}->{value} \n");
	    	$smtp->datasend("Subject: $event_priority, $log_message\n");
	    	$smtp->datasend("\n");
	    	$smtp->datasend($event_priority);
	    	$smtp->datasend("\n");
	    	$smtp->datasend($log_message);
	    	$smtp->dataend();
	    	$smtp->quit;

  	} 

	return 1;

  } # logEvent


  #
  # processEvent
  # Determine whether a ticket need to be opened, updated or closed.

  sub processEvent {

	my ($aloe, $management_ip, $component, $event_source, $threshold_values, $event_message, $event_priority) = @_;

  	my $debug	  = $aloe->{settings}->{debug}->{value};

	my $alert_path      = $aloe->{settings}->{configAlertPath}->{value};
	my $event_ticket    = join('', $alert_path, $management_ip, "-", basename($event_source), ".alert" );

	if (defined($component)) { 
		$event_ticket    = join('', $alert_path, $management_ip, ".", $component, "-", basename($event_source), ".alert" );
	}

	my $threshold_violation = thresholdViolation($threshold_values);
	my $ticket_status = "";
	my $ticket_params = "";

	my $ticket_number = "";

	#
	# Check for an existing ticket, if there is one this is either an update or a closure event.

	  if (($threshold_violation) && (open (my $tfh, "<", $event_ticket))) { close($tfh); $ticket_status = "Update"; } 

	    elsif ($threshold_violation) { $ticket_status = "New"; }

	    elsif ((!$threshold_violation) && (open (my $tfh, "<", $event_ticket))) { close($tfh); $ticket_status = "Close"; }

	if ($ticket_status) { 

		($ticket_number) = processEventTicket($aloe, $ticket_status, $event_ticket, $management_ip, $event_source, $event_message, $event_priority) ;

	 }

	return ($event_ticket, $ticket_number, $ticket_status);

  } # processEvent

  #
  # Check if any of the thresholds are in warning or error

  sub thresholdViolation {

	my ($threshold_values) = @_;

	my $threshold_violation = 0;

	foreach my $threshold_value (@$threshold_values) {


		if ($threshold_value ne "Good") { $threshold_violation = 1; }

	}

	return $threshold_violation;
  }

  #
  # Somthing was detected, generate, update or close an event ticket

  sub processEventTicket {

	my ($aloe, $ticket_status, $event_ticket, $management_ip, $event_source, $event_message, $event_priority) = @_;

  	my $debug		= $aloe->{settings}->{debug}->{value};
	my $syslog_facility     = $aloe->{settings}->{aloeLogFacility}->{value};
	my $ticket_number 	= "";

	if ($ticket_status eq "Update") {
		 my $ticket_params = XMLin($event_ticket);
		 $ticket_number    = "$ticket_params->{ticketfields}->{tnumber}->{value}";
		 updateEventTicket($aloe, $ticket_status, $ticket_number, $event_ticket, $management_ip, $event_source, $event_message, $event_priority); }

	 elsif ($ticket_status eq "New")    { $ticket_number = openEventTicket($aloe, $ticket_status, $event_ticket, $management_ip, $event_source, $event_message, $event_priority);   }

	 elsif ($ticket_status eq "Close")  {
		 my $ticket_params = XMLin($event_ticket);
		 $ticket_number    = "$ticket_params->{ticketfields}->{tnumber}->{value}";
		 closeEventTicket($aloe, $ticket_status, $ticket_number, $event_ticket, $management_ip, $event_source, $event_message, $event_priority); }

	return ($ticket_number);

  } # processEventTicket


  sub openEventTicket {

	my ($aloe, $ticket_status, $event_ticket, $management_ip, $event_source, $event_message, $event_priority) = @_;

  	my $debug		= $aloe->{settings}->{debug}->{value};
	my $syslog_facility	= $aloe->{settings}->{aloeLogFacility}->{value};
	my $aloedbhost		= $aloe->{settings}->{aloeDBHost}->{value};
	my $aloedb		= $aloe->{settings}->{aloeDB}->{value};
	my $aloedbuser		= $aloe->{settings}->{aloeDBUser}->{value};
	my $aloedbpswd		= $aloe->{settings}->{aloeDBPswd}->{value};

	my $log_message		= "$event_source $management_ip $event_message";	

	my $adbh = Mysql->connect(undef, $aloedb, $aloedbuser, $aloedbpswd);
	my $adbh = Mysql->connect($aloedbhost, $aloedb, $aloedbuser, $aloedbpswd);

  	my $dbquery = "insert into alert_logs (date, time, priority, facility, host, msg, program, status, ticket) values (now(), now(), '$event_priority', '$syslog_facility', 'localhost', '$log_message', '$event_source', '$ticket_status', '')"; 
	my $asth = $adbh->query($dbquery);
	my $ticket_number = $asth->insertid;

  	my $dbquery = "update alert_logs set ticket = $ticket_number where seq= $ticket_number"; 
	my $asth = $adbh->query($dbquery);

	if ($debug) { print "$ticket_status event ticket $ticket_number $event_ticket has been generated.\n"; }

	return $ticket_number;

  } # openEventTicket

  sub updateEventTicket {

	my ($aloe, $ticket_status, $ticket_number, $event_ticket, $management_ip, $event_source, $event_message, $event_priority) = @_;

  	my $debug		= $aloe->{settings}->{debug}->{value};
	my $syslog_facility	= $aloe->{settings}->{aloeLogFacility}->{value};
  	my $aloedbhost		= $aloe->{settings}->{aloeDBHost}->{value};
  	my $aloedb		= $aloe->{settings}->{aloeDB}->{value};
  	my $aloedbuser		= $aloe->{settings}->{aloeDBUser}->{value};
  	my $aloedbpswd		= $aloe->{settings}->{aloeDBPswd}->{value};
	my $log_message 	=  "$event_source $management_ip $event_message";	

	my $adbh = Mysql->connect(undef, $aloedb, $aloedbuser, $aloedbpswd);
	my $adbh = Mysql->connect($aloedbhost, $aloedb, $aloedbuser, $aloedbpswd);

	if ($debug) { print "Inserting a ticket update for ticket $ticket_number\n"; }

  	my $dbquery = "insert into alert_logs (date, time, priority, facility, host, msg, program, status, ticket) values (now(), now(), '$event_priority', '$syslog_facility', 'localhost', '$log_message', '$event_source', '$ticket_status', '$ticket_number')"; 
	my $asth = $adbh->query($dbquery);

  } # updateEventTicket


sub closeEventTicket {

	my ($aloe, $ticket_status, $ticket_number, $event_ticket, $management_ip, $event_source, $event_message, $event_priority) = @_;

  	my $debug		= $aloe->{settings}->{debug}->{value};
  	my $syslog_facility	= $aloe->{settings}->{aloeLogFacility}->{value};
  	my $aloedbhost		= $aloe->{settings}->{aloeDBHost}->{value};
  	my $aloedb		= $aloe->{settings}->{aloeDB}->{value};
  	my $aloedbuser		= $aloe->{settings}->{aloeDBUser}->{value};
  	my $aloedbpswd		= $aloe->{settings}->{aloeDBPswd}->{value};

	my $event_message    	= "Monitored values back within threshold, closing ticket # $ticket_number \n";
	my $log_message 	= "$event_source -> $management_ip $event_message";	
	
	my $adbh = Mysql->connect(undef, $aloedb, $aloedbuser, $aloedbpswd);
	my $adbh = Mysql->connect($aloedbhost, $aloedb, $aloedbuser, $aloedbpswd);

	if ($debug) { print "Closing ticket # $ticket_number \n"; }

	my $dbquery = "insert into alert_logs (date, time, priority, facility, host, msg, program, status, ticket) values (now(), now(), '$event_priority', '$syslog_facility', 'localhost', '$log_message', '$event_source', '$ticket_status', '$ticket_number')"; 
	my $asth = $adbh->query($dbquery);

	unlink($event_ticket);

	return ($ticket_number, $ticket_status);

#	logEvent($event_message, "notice", $management_ip);



} # closeEventTicket


return 1;